// 2008-1-16
// amotor2.c: 2 Links with an amotor by Kosei Demura 
#include <stdio.h>
#include <stdlib.h>
#include <ode/ode.h>
#include <drawstuff/drawstuff.h>

#define NUM 2
#ifdef dDOUBLE
#define dsDrawCylinder dsDrawCylinderD
#endif

dWorldID      world;
dSpaceID      space;
dGeomID       ground;
dJointID      joint[NUM], amotor;
dsFunctions fn;

typedef struct {
  dBodyID body;
  dGeomID geom;
} MyLink;

MyLink rlink[NUM];

void  createArm() 
{
  dReal init_avel = 1.0; // px[rad/s]
  dMass mass;
  dReal x[] = { 0.00, 0.00}, y[] = { 0.00, 0.00}, z[]= { 0.75 ,0.25}; // dS
  dReal length[] = { 0.50, 0.50};   // N
  dReal weight[] = { 1.00, 1.00};   // 
  dReal r[]        = { 0.02, 0.02}; // a
  dReal anchor_x[] = { 0.00, 0.00}; // ]S
  dReal anchor_y[] = { 0.00, 0.00};
  dReal anchor_z[] = { 1.00, 0.50};
  dReal axis_x[] = { 1.00, 1.00};   // ]xNg
  dReal axis_y[] = { 0.00, 0.00};
  dReal axis_z[] = { 0.00, 0.00};

  // N̐
  for (int i = 0; i < NUM; i++) { 
    rlink[i].body = dBodyCreate(world);
    dBodySetPosition(rlink[i].body, x[i], y[i], z[i]);
    dMassSetZero(&mass);
    dMassSetCylinderTotal(&mass,weight[i],3,r[i],length[i]);
    dBodySetMass(rlink[i].body, &mass);
    rlink[i].geom = dCreateCylinder(space,r[i],length[i]);
    dGeomSetBody(rlink[i].geom,rlink[i].body);
  }

  //WCg̐ƌ
  for (int j = 0; j < NUM; j++) { 
    joint[j] = dJointCreateHinge(world, 0);
		if (j == 0) dJointAttach(joint[0], rlink[0].body, 0);
    else        dJointAttach(joint[j], rlink[j].body, rlink[j-1].body);
    dJointSetHingeAnchor(joint[j], anchor_x[j], anchor_y[j], anchor_z[j]);
    dJointSetHingeAxis(joint[j], axis_x[j], axis_y[j],axis_z[j]);
  } 

  // N1ƃN2ĂWCgݒ 
  dJointSetHingeParam(joint[1], dParamLoStop, -0.0 * M_PI); // ̉
  dJointSetHingeParam(joint[1], dParamHiStop,  0.0 * M_PI); // ̏

  // amotor̐
	amotor = dJointCreateAMotor(world, 0);    // AMotor̐
  dJointAttach(amotor, rlink[0].body, 0);   // AMotorƃNƂ̌
  dJointSetAMotorMode(amotor, dAMotorUser); // [U[hɐݒAftHgAȗ
	dJointSetAMotorNumAxes(amotor, 1);        // AMotorŃRg[鎲

  // ̐ݒF]0΍WnɌŒA]xNg(1, 0, 0)
	dJointSetAMotorAxis(amotor,
	 0 /*0*/, 0 /*΍WnɌŒ*/, 1 /*x*/, 0 /*y*/, 0 /*z*/); 

  // amotor̐ݒ, pxinit_avelɐݒ
	dReal fmax = dInfinity;
  dJointSetAMotorParam(amotor, dParamVel, init_avel);
  dJointSetAMotorParam(amotor, dParamFMax, fmax);
}

void drawLink()
{
   dReal r,length;
  
   for (int i = 0; i < NUM; i++ ) { 
	  if (i == 1) dsSetColor(1.3, 1.3, 0.0);
	  else        dsSetColor(0.0, 0.0, 1.3);
    dGeomCylinderGetParams(rlink[i].geom, &r,&length);
		dsDrawCylinder(dBodyGetPosition(rlink[i].body),
		    dBodyGetRotation(rlink[i].body),length,r);
  }
}

void start()
{
  float xyz[3] = {  2,  0, 1};
  float hpr[3] = {-180, 0, 0.0};
  dsSetViewpoint(xyz,hpr);
}

// ȒPɂ邽ߏՓˌo͖
void simLoop(int pause)
{
	static int step = 0;
 
  drawLink(); 
	const dReal ang_rate = dJointGetHingeAngleRate(joint[0]);
	printf("step=%4d: angle rate=%8.6f \n", step, ang_rate);
  dWorldStep(world, 0.001);
	
  dReal k1 = 0.0; // CgNpW lݒ
	dJointAddHingeTorque(joint[0], k1 * ang_rate); // CgNt

  if (step == 0) dJointDestroy(amotor); // amotor̔j
  step++;
}

void setDrawStuff() {
  fn.version = DS_VERSION;
  fn.start   = &start;
  fn.step    = &simLoop;
  fn.path_to_textures = "../../drawstuff/textures";
}

int main(int argc, char *argv[])
{
  setDrawStuff(); 
  world = dWorldCreate();
  space = dHashSpaceCreate(0);
  // dWorldSetGravity (world, 0, 0, -9.8); // d͉x
  createArm(); 
  dsSimulationLoop(argc,argv,640, 480,&fn);
  dSpaceDestroy(space);
  dWorldDestroy(world);
  return 0;
}
